
from typing import Any, TypedDict, List, Literal, TypeAlias, Union, Protocol, runtime_checkable

__all__ = ["PreviewPayload", "PreviewPayloadInternal", "TablePreviewPayload", "TextPreviewPayload", "JSONPreviewPayload", "ImagePreviewPayload", "MediaPreviewPayload", "PandasPreviewPayload", "DefaultPreviewPayload"]

@runtime_checkable
class DataFrameIndex(Protocol):
    def tolist(self) -> Any:
        ...

# this class is for pandas.DataFrame
@runtime_checkable
class DataFrame(Protocol):

    def __len__(self) -> int:
        ...

    def __dataframe__(self, *args: Any, **kwargs: Any) -> Any:
        ...

    @property
    def index(self) -> DataFrameIndex:
        ...

    def to_csv(self, path_or_buf=None, *, sep=',', na_rep='', float_format=None, columns=None, header=True, index=True, index_label=None, mode='w', encoding=None, compression='infer', quoting=None, quotechar='"', lineterminator=None, chunksize=None, date_format=None, doublequote=True, escapechar=None, decimal='.', errors='strict', storage_options=None) -> None | str:
        ...

    def to_json(self, orient: Literal["split"]) -> str:
        ...

    def to_dict(self, orient: Literal["split"]) -> Any:
        ...

@runtime_checkable
class ShapeDataFrame(DataFrame, Protocol):

    @property
    def shape(self) -> tuple[int, int]:
        ...

@runtime_checkable
class PartialDataFrame(DataFrame, Protocol):
    def head(self, count: int) -> DataFrame:
        ...
    
    def tail(self, count: int) -> DataFrame:
        ...

class TablePreviewData(TypedDict):
    columns: List[str | int | float]
    rows: List[List[str | int | float | bool]]
    row_count: int | None

class TablePreviewPayload(TypedDict):
    type: Literal['table']
    data: TablePreviewData | Any

class TextPreviewPayload(TypedDict):
    type: Literal["text"]
    data: Any

class JSONPreviewPayload(TypedDict):
    type: Literal["json"]
    data: Any

class ImagePreviewPayload(TypedDict):
    type: Literal['image']
    data: str | List[str]

class MediaPreviewPayload(TypedDict):
    type: Literal["image", 'video', 'audio', 'markdown', "iframe", "html"]
    data: str

class PandasPreviewPayload(TypedDict):
    type: Literal['table']
    data: DataFrame

class CsvPreviewPayload(TypedDict):
    type: Literal['csv']
    data: str # csv file path

class DefaultPreviewPayload(TypedDict):
    type: str
    data: Any

PreviewPayloadInternal: TypeAlias = Union[
    TablePreviewPayload,
    TextPreviewPayload,
    JSONPreviewPayload,
    ImagePreviewPayload,
    MediaPreviewPayload,
    CsvPreviewPayload,
    PandasPreviewPayload,
    DefaultPreviewPayload
]

PreviewPayload: TypeAlias = Union[
    TablePreviewPayload,
    TextPreviewPayload,
    JSONPreviewPayload,
    ImagePreviewPayload,
    MediaPreviewPayload,
    CsvPreviewPayload,
    DataFrame,
    PandasPreviewPayload,
    DefaultPreviewPayload
]


from .mainframe import Mainframe
from .data import JobDict
import random
import string
import math
import hashlib


class InternalAPI:

    def __init__(self, client: Mainframe, job_id: JobDict) -> None:
        self._client = client
        self._job_id = job_id

    # keep this method async for future use
    async def update_node_weight(self, node_id: str, weight: int | float) -> None:
        """
        Update the weight of a node.
        :param node_id: The ID of the node to update.
        :param weight: The new weight for the node.
        """
        if not isinstance(weight, (int, float)) or not math.isfinite(weight) or weight < 0:
            raise ValueError("Weight must be a non-negative finite number.")

        self._client.send(self._job_id, {
            "type": "BlockRequest",
            "action": "UpdateNodeWeight",
            "node_id": node_id,
            "weight": weight,
            "request_id": random_string(16),
        })

def random_string(length=8):
    return ''.join(random.choices(string.ascii_letters + string.digits, k=length))


def string_hash(text: str) -> str:
    """
    Generates a deterministic hash for a given string.
    """
    return hashlib.sha256(text.encode('utf-8')).hexdigest()



import unittest
from unittest.mock import Mock
import tempfile
import os


class TestContext(unittest.TestCase):
    """Test Context class methods."""

    def setUp(self):
        """Create a Context instance with mocked dependencies."""
        from oocana import BlockInfo

        self.mock_mainframe = Mock()
        self.mock_store = {}
        self.block_info = BlockInfo(
            session_id="test_session",
            job_id="test_job",
            stacks=[{"node_id": "test_node", "flow": "test_flow"}],
            block_path="test/block"
        )
        self.temp_dir = tempfile.mkdtemp()
        self.session_dir = tempfile.mkdtemp()

        # Define outputs with different handle types
        self.outputs_def = {
            "string_output": {
                "handle": "string_output",
                "description": "A string output",
                "json_schema": {"type": "string"},
                "kind": None,
                "nullable": False,
                "is_additional": False,
            },
            "var_output": {
                "handle": "var_output",
                "description": "A var output",
                "json_schema": {"type": "object", "contentMediaType": "oomol/var"},
                "kind": None,
                "nullable": False,
                "is_additional": False,
            },
            "bin_output": {
                "handle": "bin_output",
                "description": "A binary output",
                "json_schema": {"type": "string", "contentMediaType": "oomol/bin"},
                "kind": None,
                "nullable": False,
                "is_additional": False,
            },
        }

        self.inputs_def = {}
        self.inputs = {}

    def _create_context(self):
        """Helper to create a Context instance."""
        from oocana import Context

        return Context(
            inputs=self.inputs,
            blockInfo=self.block_info,
            mainframe=self.mock_mainframe,
            store=self.mock_store,
            inputs_def=self.inputs_def,
            outputs_def=self.outputs_def,
            session_dir=self.session_dir,
            tmp_dir=self.temp_dir,
            package_name="test_package",
            pkg_dir=self.temp_dir,
        )

    def test_finish_without_result(self):
        """Test finish() without result sends BlockFinished message."""
        ctx = self._create_context()
        ctx.finish()

        self.mock_mainframe.send.assert_called_once()
        call_args = self.mock_mainframe.send.call_args
        payload = call_args[0][1]

        self.assertEqual(payload["type"], "BlockFinished")
        self.assertNotIn("result", payload)
        self.assertNotIn("error", payload)

    def test_finish_with_error(self):
        """Test finish() with error sends error message."""
        ctx = self._create_context()
        ctx.finish(error="Test error message")

        self.mock_mainframe.send.assert_called_once()
        call_args = self.mock_mainframe.send.call_args
        payload = call_args[0][1]

        self.assertEqual(payload["type"], "BlockFinished")
        self.assertEqual(payload["error"], "Test error message")

    def test_finish_with_result(self):
        """Test finish() with result sends result message."""
        ctx = self._create_context()
        ctx.finish(result={"string_output": "test value"})

        self.mock_mainframe.send.assert_called_once()
        call_args = self.mock_mainframe.send.call_args
        payload = call_args[0][1]

        self.assertEqual(payload["type"], "BlockFinished")
        self.assertEqual(payload["result"], {"string_output": "test value"})

    def test_finish_error_takes_priority(self):
        """Test finish() with both error and result only sends error."""
        ctx = self._create_context()
        ctx.finish(error="Test error", result={"string_output": "test"})

        call_args = self.mock_mainframe.send.call_args
        payload = call_args[0][1]

        self.assertEqual(payload["type"], "BlockFinished")
        self.assertEqual(payload["error"], "Test error")
        self.assertNotIn("result", payload)

    def test_output_sends_message(self):
        """Test output() sends BlockOutput message."""
        ctx = self._create_context()
        ctx.output("string_output", "hello world")

        self.mock_mainframe.send.assert_called_once()
        call_args = self.mock_mainframe.send.call_args
        payload = call_args[0][1]

        self.assertEqual(payload["type"], "BlockOutput")
        self.assertEqual(payload["handle"], "string_output")
        self.assertEqual(payload["output"], "hello world")

    def test_output_undefined_handle_warns(self):
        """Test output() with undefined handle sends warning."""
        ctx = self._create_context()
        ctx.output("undefined_handle", "value")

        # Should call report with warning, not send
        self.mock_mainframe.report.assert_called()
        call_args = self.mock_mainframe.report.call_args
        payload = call_args[0][1]

        self.assertEqual(payload["type"], "BlockWarning")
        self.assertIn("undefined_handle", payload["warning"])

    def test_outputs_sends_batch_message(self):
        """Test outputs() sends BlockOutputs message."""
        ctx = self._create_context()
        ctx.outputs({"string_output": "value1"})

        self.mock_mainframe.send.assert_called_once()
        call_args = self.mock_mainframe.send.call_args
        payload = call_args[0][1]

        self.assertEqual(payload["type"], "BlockOutputs")
        self.assertEqual(payload["outputs"], {"string_output": "value1"})

    def test_send_warning(self):
        """Test send_warning() sends BlockWarning report."""
        ctx = self._create_context()
        ctx.send_warning("Test warning message")

        self.mock_mainframe.report.assert_called_once()
        call_args = self.mock_mainframe.report.call_args
        payload = call_args[0][1]

        self.assertEqual(payload["type"], "BlockWarning")
        self.assertEqual(payload["warning"], "Test warning message")

    def test_error(self):
        """Test error() sends BlockError message."""
        ctx = self._create_context()
        ctx.error("Test error message")

        self.mock_mainframe.send.assert_called_once()
        call_args = self.mock_mainframe.send.call_args
        payload = call_args[0][1]

        self.assertEqual(payload["type"], "BlockError")
        self.assertEqual(payload["error"], "Test error message")

    def test_send_message(self):
        """Test send_message() sends BlockMessage report."""
        ctx = self._create_context()
        ctx.send_message({"custom": "payload"})

        self.mock_mainframe.report.assert_called_once()
        call_args = self.mock_mainframe.report.call_args
        payload = call_args[0][1]

        self.assertEqual(payload["type"], "BlockMessage")
        self.assertEqual(payload["payload"], {"custom": "payload"})

    def test_session_id_property(self):
        """Test session_id property returns correct value."""
        ctx = self._create_context()
        self.assertEqual(ctx.session_id, "test_session")

    def test_job_id_property(self):
        """Test job_id property returns correct value."""
        ctx = self._create_context()
        self.assertEqual(ctx.job_id, "test_job")

    def test_node_id_property(self):
        """Test node_id property returns correct value from stacks."""
        ctx = self._create_context()
        self.assertEqual(ctx.node_id, "test_node")

    def test_tmp_dir_property(self):
        """Test tmp_dir property returns correct value."""
        ctx = self._create_context()
        self.assertEqual(ctx.tmp_dir, self.temp_dir)

    def test_session_dir_property(self):
        """Test session_dir property returns correct value."""
        ctx = self._create_context()
        self.assertEqual(ctx.session_dir, self.session_dir)

    def test_inputs_def_readonly(self):
        """Test inputs_def is read-only."""
        ctx = self._create_context()
        inputs_def = ctx.inputs_def

        with self.assertRaises(TypeError):
            inputs_def["new_key"] = {}  # type: ignore

    def test_outputs_def_readonly(self):
        """Test outputs_def is read-only."""
        ctx = self._create_context()
        outputs_def = ctx.outputs_def

        with self.assertRaises(TypeError):
            outputs_def["new_key"] = {}  # type: ignore

    def tearDown(self):
        """Clean up temporary directories."""
        import shutil
        if os.path.exists(self.temp_dir):
            shutil.rmtree(self.temp_dir)
        if os.path.exists(self.session_dir):
            shutil.rmtree(self.session_dir)


if __name__ == "__main__":
    unittest.main()



from simplejson import loads
import paho.mqtt.client as mqtt
from paho.mqtt.enums import CallbackAPIVersion
import operator
from urllib.parse import urlparse
import uuid
from .data import BlockDict, JobDict, dumps, EXECUTOR_NAME
import logging
from typing import Optional, Callable, Any

__all__ = ["Mainframe"]

class Mainframe:
    address: str
    client: mqtt.Client
    client_id: str
    _subscriptions: set[str]
    _logger: logging.Logger
    __report_callbacks: set[Callable[[Any], Any]]
    __session_callbacks: dict[str, list[Callable[[dict], Any]]]
    __request_response_callbacks: dict[str, list[Callable[[dict], Any]]]

    def __init__(self, address: str, client_id: Optional[str] = None, logger = None) -> None:
        self.address = address
        self.client_id = client_id or f"python-executor-{uuid.uuid4().hex[:8]}"
        self._logger = logger or logging.getLogger(__name__)
        self._subscriptions = set()
        self.__report_callbacks = set()
        self.__session_callbacks = {}
        self.__request_response_callbacks = {}

    def connect(self):
        connect_address = (
            self.address
            if operator.contains(self.address, "://")
            else f"mqtt://{self.address}"
        )
        url = urlparse(connect_address)
        client = self._setup_client()
        client.connect(host=url.hostname, port=url.port) # type: ignore
        client.loop_start()

        def report_callback(_client, _userdata, message):
            """Internal method to handle report messages and call registered callbacks."""
            payload = loads(message.payload)
            self._logger.info("Received report: {}".format(payload))
            
            callbacks = self.__report_callbacks.copy()
            for callback in callbacks:
                try:
                    callback(payload)
                except Exception as e:
                    self._logger.error("Error in report callback: {}".format(e))

        self.client.message_callback_add("report", report_callback)
        self._subscriptions.add("report")
    
    def _setup_client(self):
        self.client = mqtt.Client(
            callback_api_version=CallbackAPIVersion.VERSION2,
            client_id=self.client_id,
        )
        self.client.logger = self._logger
        self.client.on_connect = self.on_connect
        self.client.on_disconnect = self.on_disconnect
        self.client.on_connect_fail = self.on_connect_fail # type: ignore
        return self.client

    # mqtt v5 重连后，订阅和队列信息会丢失(v3 在初始化时，设置 clean_session 后，会保留两者。
    # 我们的 broker 使用的是 v5，在 on_connect 里订阅，可以保证每次重连都重新订阅上。
    def on_connect(self, client, userdata, flags, reason_code, properties):
        if reason_code != 0:
            self._logger.error("connect to broker failed, reason_code: %s", reason_code)
            return
        else:
            self._logger.info("connect to broker success")

        for topic in self._subscriptions.copy(): # 进程冲突
            self._logger.info("resubscribe to topic: {}".format(topic))
            self.client.subscribe(topic, qos=1)

    def on_connect_fail(self) -> None:
        self._logger.error("connect to broker failed")

    def on_disconnect(self, client, userdata, flags, reason_code, properties):
        self._logger.warning("disconnect to broker, reason_code: %s", reason_code)

    # 不等待 publish 完成，使用 qos 参数来会保证消息到达。
    def send(self, job_info: JobDict, msg) -> mqtt.MQTTMessageInfo:
        return self.client.publish(
            f'session/{job_info["session_id"]}', dumps({"job_id": job_info["job_id"], "session_id": job_info["session_id"], **msg}), qos=1
        )

    def report(self, block_info: BlockDict, msg: dict) -> mqtt.MQTTMessageInfo:
        return self.client.publish("report", dumps({**block_info, **msg}), qos=1)
    
    def notify_executor_ready(self, session_id: str, package: str | None, identifier: str | None, debug_port: int | None) -> None:
        self.client.publish(f"session/{session_id}", dumps({
            "type": "ExecutorReady",
            "session_id": session_id,
            "executor_name": EXECUTOR_NAME,
            "package": package,
            "identifier": identifier,
            "debug_port": debug_port,
        }), qos=1)

    def notify_block_ready(self, session_id: str, job_id: str) -> dict:

        topic = f"inputs/{session_id}/{job_id}"
        replay = None

        def on_message_once(_client, _userdata, message):
            nonlocal replay
            self.client.unsubscribe(topic)
            replay = loads(message.payload)

        self.client.subscribe(topic, qos=1)
        self.client.message_callback_add(topic, on_message_once)

        self.client.publish(f"session/{session_id}", dumps({
            "type": "BlockReady",
            "session_id": session_id,
            "job_id": job_id,
        }), qos=1)

        while True:
            if replay is not None:
                self._logger.info("notify ready success in {} {}".format(session_id, job_id))
                return replay
            
    def add_request_response_callback(self, session_id: str, request_id: str, callback: Callable[[Any], Any]):
        """Add a callback to be called when an error occurs while running a block."""
        if not callable(callback):
            raise ValueError("Callback must be callable")
        
        if request_id not in self.__request_response_callbacks:
            self.__request_response_callbacks[request_id] = []
            self.subscribe(f"session/{session_id}/request/{request_id}/response", lambda payload: [cb(payload) for cb in self.__request_response_callbacks[request_id].copy()])

        self.__request_response_callbacks[request_id].append(callback)

    def remove_request_response_callback(self, session_id: str, request_id: str, callback: Callable[[Any], Any]):
        """Remove a previously added run block error callback."""
        if request_id in self.__request_response_callbacks and callback in self.__request_response_callbacks[request_id]:
            self.__request_response_callbacks[request_id].remove(callback)
            if len(self.__request_response_callbacks[request_id]) == 0:
                del self.__request_response_callbacks[request_id]
                self.unsubscribe(f"session/{session_id}/request/{request_id}/response")
        else:
            self._logger.warning("Callback not found in request/response callbacks for session {} and request {}.".format(session_id, request_id))

    def add_session_callback(self, session_id: str, callback: Callable[[dict], Any]):
        """Add a callback to be called when a session message is received."""
        if not callable(callback):
            raise ValueError("Callback must be callable")
        
        if session_id not in self.__session_callbacks:
            self.__session_callbacks[session_id] = []
            self.subscribe(f"session/{session_id}", lambda payload: [cb(payload) for cb in self.__session_callbacks[session_id].copy()])

        self.__session_callbacks[session_id].append(callback)

    def remove_session_callback(self, session_id: str, callback: Callable[[dict], Any]):
        """Remove a previously added session callback."""
        if session_id in self.__session_callbacks and callback in self.__session_callbacks[session_id]:
            self.__session_callbacks[session_id].remove(callback)
            if len(self.__session_callbacks[session_id]) == 0:
                del self.__session_callbacks[session_id]
                self.unsubscribe(f"session/{session_id}")
        else:
            self._logger.warning("Callback not found in session callbacks for session: {}".format(session_id))


    def add_report_callback(self, fn):
        """Add a callback to be called when a report is received."""
        if not callable(fn):
            raise ValueError("Callback must be callable")
        self.__report_callbacks.add(fn)

    def remove_report_callback(self, fn):
        """Remove a previously added report callback."""
        if fn in self.__report_callbacks:
            self.__report_callbacks.remove(fn)
        else:
            self._logger.warning("Callback not found in report callbacks")

            
    def publish(self, topic, payload):
        self.client.publish(topic, dumps(payload), qos=1)
    
    def subscribe(self, topic: str, callback):
        def on_message(_client, _userdata, message):
            self._logger.info("receive topic: {} payload: {}".format(topic, message.payload))
            payload = loads(message.payload)
            callback(payload)

        self.client.message_callback_add(topic, on_message)
        self._subscriptions.add(topic)

        if self.client.is_connected():
            self.client.subscribe(topic, qos=1)
            self._logger.info("subscribe to topic: {}".format(topic))
        else:
            self._logger.info("wait connected to subscribe to topic: {}".format(topic))


    def unsubscribe(self, topic):
        self.client.message_callback_remove(topic)
        self.client.unsubscribe(topic)
        self._subscriptions.remove(topic)

    def loop(self):
        self.client.loop_forever()

    def disconnect(self):
        self.client.disconnect()



from dataclasses import dataclass, field
from typing import Any, Optional
from .schema import FieldSchema, ContentMediaType

__all__ = ["HandleDef", "InputHandleDef", "OutputHandleDef"]

@dataclass(frozen=True, kw_only=True)
class DataDict:
    
    def get(self, key: str, default: Any = None) -> Any:
        """obj.get('key', default)"""
        return getattr(self, key, default)
    
    def __getitem__(self, key: str) -> Any:
        """obj['key']"""
        if hasattr(self, key):
            return getattr(self, key)
        raise KeyError(f"'{key}' not found")
    
    def __contains__(self, key: str) -> bool:
        """'key' in obj"""
        return hasattr(self, key)

@dataclass(frozen=True, kw_only=True)
class HandleDef(DataDict):
    """The base handle for def which is output and input handle's common part.
    """
    handle: str
    """The name of the handle. it should be unique in handle list."""

    """The description of the handle. It is used to display in the UI."""
    description: Optional[str] = None

    json_schema: Optional[FieldSchema] = None
    """The schema of the handle. It is used to validate the handle's content."""

    kind: Optional[str] = None
    """A alias of the handle's type name. It is used to display in the UI and connect to the other handle match"""

    is_additional: Optional[bool] = None
    """If the handle is an additional handle, this field is not original field, it is generated by oocana. If True, the handle is an additional handle, otherwise it is a normal handle."""

    _raw_json_schema: Optional[dict] = field(init=False, repr=False, default=None)
    """The raw json schema of the handle. Only dict can express some fields do not exist, so keep it for compatibility."""

    def __init__(self, **kwargs):
        for key, value in kwargs.items():
            object.__setattr__(self, key, value)
        if "handle" not in kwargs:
            raise ValueError("missing attr key: 'handle'")
        json_schema = self.json_schema
        if json_schema is not None and not isinstance(json_schema, FieldSchema):
            object.__setattr__(self, "_raw_json_schema", json_schema)
            object.__setattr__(self, "json_schema", FieldSchema.generate_schema(json_schema))

    def check_handle_type(self, type: ContentMediaType) -> bool:
        if self.handle is None:
            return False
        if self.json_schema is None:
            return False
        if self.json_schema.contentMediaType is None:
            return False
        return self.json_schema.contentMediaType == type

    def json_schema_to_dict(self) -> Optional[dict]:
        """Return the raw JSON schema as a dictionary, or None if no schema exists."""
        return self._raw_json_schema if self._raw_json_schema else None

    def is_var_handle(self) -> bool:
        return self.check_handle_type("oomol/var")
    
    def is_secret_handle(self) -> bool:
        return self.check_handle_type("oomol/secret")

    def is_bin_handle(self) -> bool:
        return self.check_handle_type("oomol/bin")
    
    def is_credential_handle(self) -> bool:
        return self.check_handle_type("oomol/credential")
    
    def is_additional_handle(self) -> bool:
        """Check if the handle is an additional handle."""
        return hasattr(self, "is_additional") and self.is_additional is True

@dataclass(frozen=True, kw_only=True)
class InputHandleDef(HandleDef):

    value: Optional[Any] = None
    """default value for input handle, can be None.
    """

    nullable: Optional[bool] = None
    """If the handle value can be None. If True, the handle can be None, otherwise it must have a value."""

    _deserialize_from_cache = False
    """If True, the handle will be serialized for cache. If False, the handle will not be serialized for cache. only work for var handle and the var is pandas DataFrame."""

    _has_value: bool = field(init=False, repr=False, default=False)
    """If True, the handle has a value. If False, the handle does not have a value."""

    def is_serializable_var(self) -> bool:
        """Check if the input handle is a serializable variable."""
        return self.is_var_handle() and self._deserialize_from_cache is True
    
    def has_value(self) -> bool:
        """Check if the input handle has a value."""
        return self._has_value

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        if "value" in kwargs:
            object.__setattr__(self, "_has_value", True)
        else:
            object.__setattr__(self, "_has_value", False)


@dataclass(frozen=True, kw_only=True)
class OutputHandleDef(HandleDef):

    _serialize_for_cache: Optional[bool] = None
    """If True, the handle will be serialized for cache. If False, the handle will not be serialized for cache."""

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def need_serialize_var_for_cache(self) -> bool:
        return self.is_var_handle() and self._serialize_for_cache is True


import unittest
import timeit
from oocana import handle  # 假设 handle_data 包含 HandleDef dataclass


class TestPerformance(unittest.TestCase):
    # 定义一个小的测试函数，用于生成和初始化 HandleDef 实例
    def test_handle_def_creation(self):
        d = {
            "handle": "test",
            "json_schema": {
                "contentMediaType": "oomol/bin"
            },
        }
        handle_def = handle.HandleDef(**d)
        assert(handle_def.handle == "test")


if __name__ == "__main__":
    # 使用 timeit 模块测量执行时间
    execution_time = timeit.timeit("test_handle_def_creation()", globals=globals(), number=10000)
    print(f"HandleDef creation and initialization took {execution_time} seconds for 10000 iterations")


import simplejson as sjson
import json
import math

def test_json():
    data = {
        "name": "John",
        "age": 30,
        "city": "New York"
    }

    # use the json module
    json_data = json.dumps(data)
    assert json_data == '{"name": "John", "age": 30, "city": "New York"}'

    # use the simplejson module
    sjson_data = sjson.dumps(data)
    assert sjson_data == '{"name": "John", "age": 30, "city": "New York"}'

    # use the json module
    json_data = json.loads(json_data)
    assert json_data == data

    # use the simplejson module
    sjson_data = sjson.loads(sjson_data)
    assert sjson_data == data

def test_none_in_json():
    data = {
        "name": "John",
        "age": None,
        "city": "New York"
    }

    # use the json module
    json_data = json.dumps(data)
    assert json_data == '{"name": "John", "age": null, "city": "New York"}'

    # use the simplejson module
    sjson_data = sjson.dumps(data)
    assert sjson_data == '{"name": "John", "age": null, "city": "New York"}'

    # use the json module
    json_data = json.loads(json_data)
    assert json_data == data

    # use the simplejson module
    sjson_data = sjson.loads(sjson_data)
    assert sjson_data == data

def test_nan_in_json():
    data = {
        "name": "John",
        "age": float('nan'),
        "city": "New York"
    }

    # use the json module
    json_data = json.dumps(data)
    assert json_data == '{"name": "John", "age": NaN, "city": "New York"}'

    # use the simplejson module
    sjson_data = sjson.dumps(data, ignore_nan=True)
    assert sjson_data == '{"name": "John", "age": null, "city": "New York"}'

    # use the json module
    json_data = json.loads(json_data)
    assert math.isnan(json_data['age'])

    # use the simplejson module
    sjson_data = sjson.loads(sjson_data)
    assert sjson_data.get('age') is None